==> cx88-1.4.4.tar.gz: cx88-1.4.4/client/drivers/wintv_hvr1800.cpp <==
/*-
 * Copyright 2009 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/errno.h>
#include "mt2131.h"
#include "cx24227.h"
#include "tda8295.h"
#include "cx88_debug.h"
#include "wintv_hvr1800.h"

namespace cx88_mpeg_driver_factory
{

   const cx88_source wintv_hvr1800::m_sources[] = 
      {{"Primary",   "Main digital tuner",   cx88_driver::CX88_SOURCE_DEFAULT},
       {"Secondary", "Digital/analog tuner", wintv_hvr1800::HVR1800_SOURCE_SECONDARY},
       {NULL,        NULL,                   0}};

   const cx88_source *wintv_hvr1800::get_sources(void)
   {
      return ((m_tuners == HVR1800_TUNER_MICROTUNE_NXP) ? m_sources : cx88_driver::get_sources());
   }

   int wintv_hvr1800::tda18271_interface_config(tda18271 &driver, const dvb_channel &channel, tda18271_interface &ifc)
   {
      ifc.std = 0x00;
      ifc.if_level = 0x06;
      switch (channel.modulation)
      {
         case DVB_MOD_VSB_8:
         case DVB_MOD_VSB_16:
            ifc.ifreq_hz = 5380000;
            break;
         default:
            break;
      }
      return 0;
   }

   int wintv_hvr1800::create_i2c_drivers(void)
   {
      cx88_common_gpio gpio;
      gpio.in_write_mask = 0x4000;
      gpio.in_read_mask = 0x0;
      gpio.inout_gpio = 0x0;
      if (ioctl(m_fd, CX23885_COMMON_MC_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during microcontroller GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(100000);
      gpio.inout_gpio = 0x4000;
      if (ioctl(m_fd, CX23885_COMMON_MC_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during microcontroller GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(100000);
      int error = 0;
      m_i2c_devs[3].driver = new cx24227(m_tunerconf, *(m_i2c_devs[3].dev), DVB_INPUT_SERIAL, 
         CX24227_QAM_IFREQ_44MHZ, CX24227_GPIO_ENABLE, CX24227_CLK_CONT_NINV, error);
      m_i2c_devs[2].driver = new mt2131(m_tunerconf, *(m_i2c_devs[2].dev), error);
      if ((m_i2c_devs[2].driver == NULL) || (m_i2c_devs[3].driver == NULL))
      {
         error = ENOMEM;
      }
      if (error)
      {
         CX88ERR << "Error initializing primary tuner/demodulator driver for WinTV HVR-1800" << std::endl;
         return error;
      }
      return 0;
   }

   void wintv_hvr1800::destroy_i2c_drivers(void)
   {
      delete m_i2c_devs[0].driver;
      delete m_i2c_devs[1].driver;
      delete m_i2c_devs[2].driver;
      delete m_i2c_devs[3].driver;
   }

   int wintv_hvr1800::create_i2c_devices(void)
   {
      if (m_tuners == HVR1800_TUNER_MICROTUNE_NXP)
      {
         m_i2c_devs[0].bus = 1;
         m_i2c_devs[0].conf.in_addr = 0x60;
         m_i2c_devs[0].conf.in_speed = CX88_I2C_SPEED_400KHZ;
         m_i2c_devs[0].source_mask = HVR1800_SOURCE_SECONDARY;
         m_i2c_devs[1].bus = 1;
         m_i2c_devs[1].conf.in_addr = 0x42;
         m_i2c_devs[1].conf.in_speed = CX88_I2C_SPEED_400KHZ;
         m_i2c_devs[1].source_mask = 0;
      }
      m_i2c_devs[2].bus = 0;
      m_i2c_devs[2].conf.in_addr = 0x61;
      m_i2c_devs[2].conf.in_speed = CX88_I2C_SPEED_400KHZ;
      m_i2c_devs[2].source_mask = CX88_SOURCE_DEFAULT;
      m_i2c_devs[3].bus = 0;
      m_i2c_devs[3].conf.in_addr = 0x19;
      m_i2c_devs[3].conf.in_speed = CX88_I2C_SPEED_400KHZ;
      m_i2c_devs[3].source_mask = CX88_SOURCE_DEFAULT | HVR1800_SOURCE_SECONDARY;
      return 0;
   }

   int wintv_hvr1800::select_source(uint32_t source_mask)
   {
      if (m_current_source->mask == source_mask)
      {
         return 0;
      }
      int error = 0;
      if (source_mask == HVR1800_SOURCE_SECONDARY)
      {
         cx88_common_gpio gpio;
         gpio.in_write_mask = 0x4;
         gpio.in_read_mask = 0x0;
         gpio.inout_gpio = 0x4;
         if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
         {
            CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
            return errno;
         }
         usleep(20000);
         gpio.inout_gpio = 0x0;
         if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
         {
            CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
            return errno;
         }
         usleep(20000);
         gpio.inout_gpio = 0x4;
         if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
         {
            CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
            return errno;
         }
         usleep(20000);
         delete m_i2c_devs[3].driver;
         m_i2c_devs[3].driver = new cx24227(m_tunerconf, *(m_i2c_devs[3].dev), DVB_INPUT_SERIAL, 
            CX24227_QAM_IFREQ_4MHZ, CX24227_GPIO_DISABLE, CX24227_CLK_CONT_NINV, error);   
         if (m_i2c_devs[1].driver == NULL)
         {
            m_i2c_devs[1].driver = new tda8295(m_tunerconf, *(m_i2c_devs[1].dev), error);
         }
         if (m_i2c_devs[0].driver == NULL)
         {
            m_i2c_devs[0].driver = new tda18271(m_tunerconf, *(m_i2c_devs[0].dev), TDA18271_MODE_SINGLE,
            NULL, tda18271_interface_config, error);
         }
         if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL) || (m_i2c_devs[3].driver == NULL))
         {
            error = ENOMEM;
         }
         if (error)
         {
            CX88ERR << "Error initializing secondary tuner/demodulator driver for WinTV HVR-1800" << std::endl;
            return error;
         }
      }
      else
      {
         delete m_i2c_devs[3].driver;
         delete m_i2c_devs[1].driver;
         delete m_i2c_devs[0].driver;
         m_i2c_devs[1].driver = NULL;
         m_i2c_devs[0].driver = NULL;
         m_i2c_devs[3].driver = new cx24227(m_tunerconf, *(m_i2c_devs[3].dev), DVB_INPUT_SERIAL, 
            CX24227_QAM_IFREQ_44MHZ, CX24227_GPIO_ENABLE, CX24227_CLK_CONT_NINV, error);
         if (m_i2c_devs[3].driver == NULL)
         {
            return ENOMEM;
         }
      }
      return error;
   }

   int wintv_hvr1800::set_channel_options(dvb_channel &channel) 
   {
      if ((m_source->mask == HVR1800_SOURCE_SECONDARY) && 
         ((channel.modulation == DVB_MOD_VSB_8) || (channel.inversion == DVB_INVERSION_AUTO)))
      {
         channel.inversion = DVB_INVERSION_ON;
      }
      return 0;
   }
}

namespace cx88_analog_driver_factory
{
   const cx88_source wintv_hvr1800::m_sources[] = 
      {{"Tuner",     "Analog tuner",       cx88_driver::CX88_SOURCE_DEFAULT},
       {"Line-In",   "Line-In audio jack", wintv_hvr1800::HVR1800_SOURCE_LINEIN},
       {NULL,        NULL,                 0}};

   int wintv_hvr1800::create_i2c_devices(void)
   {
      m_i2c_devs[0].bus = 1;
      m_i2c_devs[0].conf.in_addr = 0x60;
      m_i2c_devs[0].conf.in_speed = CX88_I2C_SPEED_400KHZ;
      m_i2c_devs[0].source_mask = CX88_SOURCE_DEFAULT;
      m_i2c_devs[1].bus = 1;
      m_i2c_devs[1].conf.in_addr = 0x42;
      m_i2c_devs[1].conf.in_speed = CX88_I2C_SPEED_400KHZ;
      m_i2c_devs[1].source_mask = CX88_SOURCE_DEFAULT;
      return 0;
   }

   int wintv_hvr1800::create_i2c_drivers(void)
   {
      cx88_common_gpio gpio;
      int error = 0;
      gpio.in_write_mask = 0x4;
      gpio.in_read_mask = 0x0;
      gpio.inout_gpio = 0x4;
      if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(20000);
      gpio.inout_gpio = 0x0;
      if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(20000);
      gpio.inout_gpio = 0x4;
      if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(20000);
      m_i2c_devs[1].driver = new tda8295(m_tunerconf, *(m_i2c_devs[1].dev), error);
      m_i2c_devs[0].driver = new tda18271(m_tunerconf, *(m_i2c_devs[0].dev), TDA18271_MODE_SINGLE,
         NULL, NULL, error);
      if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL))
      {
         error = ENOMEM;
      }
      if (error)
      {
         CX88ERR << "Error initializing secondary tuner/demodulator driver for WinTV HVR-1800" << std::endl;
         return error;
      }
      return 0;
   }

   void wintv_hvr1800::destroy_i2c_drivers(void)
   {
      delete m_i2c_devs[0].driver;
      delete m_i2c_devs[1].driver;
   }

   int wintv_hvr1800::select_source(uint32_t source_mask)
   {
      m_audio_input = CX23885_AUDIO_INPUT_BCAST2;
      if (source_mask == HVR1800_SOURCE_LINEIN)
      {
         m_audio_input = CX23885_AUDIO_INPUT_AC97;
      }
      return 0;
   }
   
   int wintv_hvr1800::set_format_options(cx88_audio_format &format)
   {
      format.in_input = m_audio_input;
      return 0;
   }

}

==> cx88-1.4.6.tar.gz: cx88-1.4.6/client/drivers/wintv_hvr1800.cpp <==
/*-
 * Copyright 2009 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/errno.h>
#include <new>
#include "mt2131.h"
#include "cx24227.h"
#include "tda8295.h"
#include "cx88_debug.h"
#include "wintv_hvr1800.h"

namespace cx88_mpeg_driver_factory
{

   const cx88_source wintv_hvr1800::m_sources[] = 
      {{"Primary",   "Main digital tuner",   cx88_driver::CX88_SOURCE_DEFAULT},
       {"Secondary", "Digital/analog tuner", wintv_hvr1800::HVR1800_SOURCE_SECONDARY},
       {NULL,        NULL,                   0}};

   const cx88_source *wintv_hvr1800::get_sources(void)
   {
      return ((m_tuners == HVR1800_TUNER_MICROTUNE_NXP) ? m_sources : cx88_driver::get_sources());
   }

   int wintv_hvr1800::tda18271_interface_config(tda18271 &driver, const dvb_channel &channel, tda18271_interface &ifc)
   {
      ifc.std = 0x00;
      ifc.if_level = 0x06;
      switch (channel.modulation)
      {
         case DVB_MOD_VSB_8:
         case DVB_MOD_VSB_16:
            ifc.ifreq_hz = 5380000;
            break;
         default:
            break;
      }
      return 0;
   }

   int wintv_hvr1800::create_i2c_drivers(void)
   {
      cx88_common_gpio gpio;
      gpio.in_write_mask = 0x4000;
      gpio.in_read_mask = 0x0;
      gpio.inout_gpio = 0x0;
      if (ioctl(m_fd, CX23885_COMMON_MC_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during microcontroller GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(100000);
      gpio.inout_gpio = 0x4000;
      if (ioctl(m_fd, CX23885_COMMON_MC_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during microcontroller GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(100000);
      int error = 0;
      m_i2c_devs[3].driver = new(std::nothrow) cx24227(m_tunerconf, *(m_i2c_devs[3].dev), DVB_INPUT_SERIAL, 
         CX24227_QAM_IFREQ_44MHZ, CX24227_GPIO_ENABLE, CX24227_CLK_CONT_NINV, error);
      m_i2c_devs[2].driver = new(std::nothrow) mt2131(m_tunerconf, *(m_i2c_devs[2].dev), error);
      if ((m_i2c_devs[2].driver == NULL) || (m_i2c_devs[3].driver == NULL))
      {
         error = ENOMEM;
      }
      if (error)
      {
         CX88ERR << "Error initializing primary tuner/demodulator driver for WinTV HVR-1800" << std::endl;
         return error;
      }
      return 0;
   }

   void wintv_hvr1800::destroy_i2c_drivers(void)
   {
      delete m_i2c_devs[0].driver;
      delete m_i2c_devs[1].driver;
      delete m_i2c_devs[2].driver;
      delete m_i2c_devs[3].driver;
   }

   int wintv_hvr1800::create_i2c_devices(void)
   {
      if (m_tuners == HVR1800_TUNER_MICROTUNE_NXP)
      {
         m_i2c_devs[0].bus = 1;
         m_i2c_devs[0].conf.in_addr = 0x60;
         m_i2c_devs[0].conf.in_speed = CX88_I2C_SPEED_400KHZ;
         m_i2c_devs[0].source_mask = HVR1800_SOURCE_SECONDARY;
         m_i2c_devs[1].bus = 1;
         m_i2c_devs[1].conf.in_addr = 0x42;
         m_i2c_devs[1].conf.in_speed = CX88_I2C_SPEED_400KHZ;
         m_i2c_devs[1].source_mask = 0;
      }
      m_i2c_devs[2].bus = 0;
      m_i2c_devs[2].conf.in_addr = 0x61;
      m_i2c_devs[2].conf.in_speed = CX88_I2C_SPEED_400KHZ;
      m_i2c_devs[2].source_mask = CX88_SOURCE_DEFAULT;
      m_i2c_devs[3].bus = 0;
      m_i2c_devs[3].conf.in_addr = 0x19;
      m_i2c_devs[3].conf.in_speed = CX88_I2C_SPEED_400KHZ;
      m_i2c_devs[3].source_mask = CX88_SOURCE_DEFAULT | HVR1800_SOURCE_SECONDARY;
      return 0;
   }

   int wintv_hvr1800::select_source(uint32_t source_mask)
   {
      if (m_current_source->mask == source_mask)
      {
         return 0;
      }
      int error = 0;
      if (source_mask == HVR1800_SOURCE_SECONDARY)
      {
         cx88_common_gpio gpio;
         gpio.in_write_mask = 0x4;
         gpio.in_read_mask = 0x0;
         gpio.inout_gpio = 0x4;
         if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
         {
            CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
            return errno;
         }
         usleep(20000);
         gpio.inout_gpio = 0x0;
         if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
         {
            CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
            return errno;
         }
         usleep(20000);
         gpio.inout_gpio = 0x4;
         if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
         {
            CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
            return errno;
         }
         usleep(20000);
         delete m_i2c_devs[3].driver;
         m_i2c_devs[3].driver = new(std::nothrow) cx24227(m_tunerconf, *(m_i2c_devs[3].dev), DVB_INPUT_SERIAL, 
            CX24227_QAM_IFREQ_4MHZ, CX24227_GPIO_DISABLE, CX24227_CLK_CONT_NINV, error);   
         if (m_i2c_devs[1].driver == NULL)
         {
            m_i2c_devs[1].driver = new(std::nothrow) tda8295(m_tunerconf, *(m_i2c_devs[1].dev), error);
         }
         if (m_i2c_devs[0].driver == NULL)
         {
            m_i2c_devs[0].driver = new(std::nothrow) tda18271(m_tunerconf, *(m_i2c_devs[0].dev), TDA18271_MODE_SINGLE,
            NULL, tda18271_interface_config, error);
         }
         if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL) || (m_i2c_devs[3].driver == NULL))
         {
            error = ENOMEM;
         }
         if (error)
         {
            CX88ERR << "Error initializing secondary tuner/demodulator driver for WinTV HVR-1800" << std::endl;
            return error;
         }
      }
      else
      {
         delete m_i2c_devs[3].driver;
         delete m_i2c_devs[1].driver;
         delete m_i2c_devs[0].driver;
         m_i2c_devs[1].driver = NULL;
         m_i2c_devs[0].driver = NULL;
         m_i2c_devs[3].driver = new(std::nothrow) cx24227(m_tunerconf, *(m_i2c_devs[3].dev), DVB_INPUT_SERIAL, 
            CX24227_QAM_IFREQ_44MHZ, CX24227_GPIO_ENABLE, CX24227_CLK_CONT_NINV, error);
         if (m_i2c_devs[3].driver == NULL)
         {
            return ENOMEM;
         }
      }
      return error;
   }

   int wintv_hvr1800::set_channel_options(dvb_channel &channel) 
   {
      if ((m_source->mask == HVR1800_SOURCE_SECONDARY) && 
         ((channel.modulation == DVB_MOD_VSB_8) || (channel.inversion == DVB_INVERSION_AUTO)))
      {
         channel.inversion = DVB_INVERSION_ON;
      }
      return 0;
   }
}

namespace cx88_analog_driver_factory
{
   const cx88_source wintv_hvr1800::m_sources[] = 
      {{"Tuner",     "Analog tuner",       cx88_driver::CX88_SOURCE_DEFAULT},
       {"Line-In",   "Line-In audio jack", wintv_hvr1800::HVR1800_SOURCE_LINEIN},
       {NULL,        NULL,                 0}};

   int wintv_hvr1800::create_i2c_devices(void)
   {
      m_i2c_devs[0].bus = 1;
      m_i2c_devs[0].conf.in_addr = 0x60;
      m_i2c_devs[0].conf.in_speed = CX88_I2C_SPEED_400KHZ;
      m_i2c_devs[0].source_mask = CX88_SOURCE_DEFAULT;
      m_i2c_devs[1].bus = 1;
      m_i2c_devs[1].conf.in_addr = 0x42;
      m_i2c_devs[1].conf.in_speed = CX88_I2C_SPEED_400KHZ;
      m_i2c_devs[1].source_mask = CX88_SOURCE_DEFAULT;
      return 0;
   }

   int wintv_hvr1800::create_i2c_drivers(void)
   {
      cx88_common_gpio gpio;
      int error = 0;
      gpio.in_write_mask = 0x4;
      gpio.in_read_mask = 0x0;
      gpio.inout_gpio = 0x4;
      if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(20000);
      gpio.inout_gpio = 0x0;
      if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(20000);
      gpio.inout_gpio = 0x4;
      if (ioctl(m_fd, CX88_COMMON_GPIO, &gpio) != 0)
      {
         CX88ERR << "Error during GPIO ioctl: " << strerror(errno) << std::endl;
         return errno;
      }
      usleep(20000);
      m_i2c_devs[1].driver = new(std::nothrow) tda8295(m_tunerconf, *(m_i2c_devs[1].dev), error);
      m_i2c_devs[0].driver = new(std::nothrow) tda18271(m_tunerconf, *(m_i2c_devs[0].dev), TDA18271_MODE_SINGLE,
         NULL, NULL, error);
      if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL))
      {
         error = ENOMEM;
      }
      if (error)
      {
         CX88ERR << "Error initializing secondary tuner/demodulator driver for WinTV HVR-1800" << std::endl;
         return error;
      }
      return 0;
   }

   void wintv_hvr1800::destroy_i2c_drivers(void)
   {
      delete m_i2c_devs[0].driver;
      delete m_i2c_devs[1].driver;
   }

   int wintv_hvr1800::select_source(uint32_t source_mask)
   {
      m_audio_input = CX23885_AUDIO_INPUT_BCAST2;
      if (source_mask == HVR1800_SOURCE_LINEIN)
      {
         m_audio_input = CX23885_AUDIO_INPUT_AC97;
      }
      return 0;
   }
   
   int wintv_hvr1800::set_format_options(cx88_audio_format &format)
   {
      format.in_input = m_audio_input;
      return 0;
   }

}
